#if IG_C111 || IG_C301 // Auto generated by AddMacroForInstantGameFiles.exe

using System;
using System.Collections.Generic;
using System.Text;
using Unity.AutoStreaming.CloudContentDelivery;
using UnityEditor;
using UnityEngine;

namespace Unity.AutoStreaming
{
    internal struct SimpleBadge
    {
        public string Name { get; set; }
        public string ReleaseId { get; set; }
        public bool Locked { get; set; }
    }

    internal struct SimpleBucket
    {
        public string Name { get; set; }
        public string Id { get; set; }
    }

    internal static class ASUOSHelper
    {
        internal static SimpleBucket[] LoadBuckets()
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return null;

            Bucket[] buckets = Bucket.LoadBuckets(UosAuthorizationToken);
            if (buckets == null)
                return null;

            SimpleBucket[] smbuckets = new SimpleBucket[buckets.Length];
            for (int i = 0; i < buckets.Length; i++)
            {
                smbuckets[i].Id = buckets[i].id;
                smbuckets[i].Name = buckets[i].name;
            }
            return smbuckets;
        }

        internal static void CreateBucket(string bucketName, string des)
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return;

            Bucket.CreateBucket(UosAuthorizationToken, bucketName, des);
        }

        internal static SimpleBadge[] LoadBadges()
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return null;

            if (string.IsNullOrEmpty(BucketUuid))
                return null;

            Badge[] badges = Badge.LoadBadges(UosAuthorizationToken, BucketUuid);
            if (badges == null)
                return null;

            SimpleBadge[] smBadges = new SimpleBadge[badges.Length];
            for (int i = 0; i < badges.Length; i++)
            {
                smBadges[i].Name = badges[i].name;
                smBadges[i].ReleaseId = badges[i].releaseid;
                smBadges[i].Locked = badges[i].islocked.Equals("true");
            }
            return smBadges;
        }

        internal static void LockBadge(string badgeName, string releaseId, bool toLock)
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return;

            if (string.IsNullOrEmpty(BucketUuid))
                return;

            Badge.LockBadge(UosAuthorizationToken, BucketUuid, badgeName, releaseId, toLock);
        }

        internal static string GetLatestReleaseId()
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return null;

            if (string.IsNullOrEmpty(BucketUuid))
                return null;

            Badge[] badges = Badge.LoadBadges(UosAuthorizationToken, BucketUuid);
            if (badges == null)
                return null;


            for (int i = 0; i < badges.Length; i++)
            {
                if (badges[i].name.Equals("latest"))
                    return badges[i].releaseid;
            }
            return null;
        }

        internal static bool SyncEntries(List<System.Tuple<ASResourceType, string>> toBeUploaded)
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return false;

            return Entry.SyncEntries(UosAppId, UosAuthorizationToken, BucketUuid, toBeUploaded);
        }

        internal static string CreateRelease(string des)
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return null;

            return Release.CreateRelease(UosAuthorizationToken, BucketUuid, des);
        }

        internal static void CreateOrUpdateBadge(string badgeName, string releaseid)
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return;

            Badge.CreateOrUpdateBadge(UosAuthorizationToken, BucketUuid, badgeName, releaseid);
        }

        internal static Project LoadProject()
        {
            if (string.IsNullOrEmpty(UosAuthorizationToken))
                return null;

            return Project.LoadProject(UosAuthorizationToken);
            //if (project != null)
            //{
            //    SimpleProject smProject = new SimpleProject();
            //    smProject.Provider = project.Provider;
            //    smProject.Status = project.Status;
            //    return smProject;
            //}
            //return null;
        }

        internal static bool IsAnOnlineVersion(string badgeName, out string tagStr)
        {
            if (BucketUuid.Equals(ByteDanceVersions.CurrentVersion.bucketUuid) && badgeName.Equals(ByteDanceVersions.CurrentVersion.badgeName))
            {
                tagStr = "- 字节上线版本";
                if (badgeName.Equals("latest"))
                    tagStr += "- 警告：latest badge不应作为上线版本使用";
                return true;
            }

            if (BucketUuid.Equals(ByteDanceVersions.AuditVersion.bucketUuid) && badgeName.Equals(ByteDanceVersions.AuditVersion.badgeName))
            {
                tagStr = "- 字节提审版本";
                if (badgeName.Equals("latest"))
                    tagStr += "- 警告：latest badge不应作为提审版本使用";

                return true;
            }

            if (BucketUuid.Equals(ByteDanceVersions.LatestVersion.bucketUuid) && badgeName.Equals(ByteDanceVersions.LatestVersion.badgeName))
            {
                tagStr = "- 字节测试版本";
                if (badgeName.Equals("latest"))
                    tagStr += "- 提示：latest badge不能转为提审和上线版本，请注意更换badge";
                return false;
            }

            tagStr = string.Empty;
            return false;
        }

        internal static string TtAppid
        {
            get
            {
                return AutoStreamingSettings.GetTtAppid();
            }

            set
            {
                AutoStreamingSettings.SetTtAppid(value);
            }
        }

        internal static string TargetPlatform
        {
            get
            {
                #if UNITY_ANDROID
                return AutoStreamingSettings.GetTargetPlaftorm();
                #else
                return "";
                #endif
            }

            set
            {
                AutoStreamingSettings.SetTargetPlaftorm(value);
            }
        }
        
        internal static string UosAuthorizationToken
        {
            get
            {
                if (string.IsNullOrEmpty(UosAppId) || string.IsNullOrEmpty(UosAppSecret) || UosAppId.Length != 36)
                {
                    return "";
                } 
                return Convert.ToBase64String(Encoding.ASCII.GetBytes(UosAppId + ":" + UosAppSecret));
            }
        }

        internal static string UosAppSecret
        {
            get
            {
                return AutoStreamingSettings.GetUosSecret();
            }
            set
            {
                AutoStreamingSettings.SetUosSecret(value);
            }
        }

        internal static string UosAppId
        {
            get
            {
                return AutoStreamingSettings.GetUosAppid();
            }
            set
            {
                AutoStreamingSettings.SetUosAppid(value);
                PlayerSettings.instantGameAppId = value;
            }
        }

        internal static string BucketUuid
        {
            get
            {
                return AutoStreamingSettings.GetBucketUuid();
            }
            set
            {
                AutoStreamingSettings.SetBucketUuid(value);
            }
        }

        internal static string BucketName
        {
            get
            {
                return AutoStreamingSettings.GetBucketName();
            }
            set
            {
                AutoStreamingSettings.SetBucketName(value);
            }
        }

        internal static string BadgeName
        {
            get
            {
                return AutoStreamingSettings.GetBadgeName();
            }
            set
            {
                AutoStreamingSettings.SetBadgeName(value);
            }
        }

        internal static string AutoStreamingPath 
        {
            get 
            {
                return string.Format("{0}client_api/v1/buckets/{1}/release_by_badge/{2}/content", UOSInfo.ApiHost, ASUOSHelper.BucketUuid, ASUOSHelper.BadgeName);
            }
        }
    }
}

#endif  // IG_C111 || IG_C301, Auto generated by AddMacroForInstantGameFiles.exe
