#if IG_C111 || IG_C301 // Auto generated by AddMacroForInstantGameFiles.exe

using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using UnityEditor.IMGUI.Controls;
using System.Linq;
using UnityEngine.Assertions;

namespace Unity.AutoStreaming
{
    internal class ASSceneTreeDataItem : TreeDataItem
    {
        public AutoStreamingSettingsScene SceneSettings { get; set; }
        public ASSceneTreeDataItem(AutoStreamingSettingsScene inSettings, int depth, int id)
            : base(inSettings.assetPath, depth, id)
        {
            SceneSettings = inSettings;
        }

        public ASSceneTreeDataItem(string name, int depth, int id)
            : base(name, depth, id)
        {
        }
    }

    internal class ASScenesTreeView : TreeViewBaseT<ASSceneTreeDataItem>
    {
        enum MyColumns
        {
            AssetPath,
            OnDemandDownload,
            AssetBundle,
        }
        enum SortOption
        {
            AssetPath,
            OnDemandDownload,
            AssetBundle,
        }

        SortOption[] m_SortOptions =
        {
            SortOption.AssetPath,
            SortOption.OnDemandDownload,
            SortOption.AssetBundle,
        };

        public ASScenesTreeView(TreeViewState state, MultiColumnHeader multicolumnHeader, TreeModelT<ASSceneTreeDataItem> model) : base(state, multicolumnHeader, model)
        {
            // Custom setup
            rowHeight = k_RowHeights;

            columnIndexForTreeFoldouts = 0;
            showAlternatingRowBackgrounds = true;
            showBorder = true;
            customFoldoutYOffset = (k_RowHeights - EditorGUIUtility.singleLineHeight) * 0.5f; // center foldout in the row since we also center content. See RowGUI

            multicolumnHeader.sortingChanged += OnSortingChanged;

            Reload();
        }

        protected override void SortByMultipleColumns()
        {
            var sortedColumns = multiColumnHeader.state.sortedColumns;

            if (sortedColumns.Length == 0)
                return;

            var myTypes = rootItem.children.Cast<TreeViewItemBaseT<ASSceneTreeDataItem>>();
            var orderedQuery = InitialOrder(myTypes, sortedColumns);
            for (int i = 1; i < sortedColumns.Length; i++)
            {
                SortOption sortOption = m_SortOptions[sortedColumns[i]];
                bool ascending = multiColumnHeader.IsSortedAscending(sortedColumns[i]);

                switch (sortOption)
                {
                    case SortOption.AssetPath:
                        orderedQuery = orderedQuery.ThenBy(l => l.Data.Name, ascending);
                        break;
                    case SortOption.AssetBundle:
                        orderedQuery = orderedQuery.ThenBy(l => l.Data.SceneSettings.assetBundleSize, ascending);
                        break;
                    case SortOption.OnDemandDownload:
                        orderedQuery = orderedQuery.ThenBy(l => l.Data.SceneSettings.onDemandDownload, ascending);
                        break;
                }
            }

            rootItem.children = orderedQuery.Cast<TreeViewItem>().ToList();
        }

        IOrderedEnumerable<TreeViewItemBaseT<ASSceneTreeDataItem>> InitialOrder(IEnumerable<TreeViewItemBaseT<ASSceneTreeDataItem>> myTypes, int[] history)
        {
            SortOption sortOption = m_SortOptions[history[0]];
            bool ascending = multiColumnHeader.IsSortedAscending(history[0]);
            switch (sortOption)
            {
                case SortOption.AssetPath:
                    return myTypes.Order(l => l.Data.Name, ascending);
                case SortOption.AssetBundle:
                    return myTypes.Order(l => l.Data.SceneSettings.assetBundleSize, ascending);
                case SortOption.OnDemandDownload:
                    return myTypes.Order(l => l.Data.SceneSettings.onDemandDownload, ascending);
                default:
                    Assert.IsTrue(false, "Unhandled enum");
                    break;
            }

            // default
            return myTypes.Order(l => l.Data.Name, ascending);
        }

        public static MultiColumnHeaderState CreateDefaultMultiColumnHeaderState(float treeViewWidth)
        {
            var columns = new[]
            {
                new MultiColumnHeaderState.Column
                {
                    headerContent = new GUIContent("AssetPath"),
                    headerTextAlignment = TextAlignment.Left,
                    sortedAscending = true,
                    sortingArrowAlignment = TextAlignment.Center,
                    width = 300,
                    minWidth = 100,
                    autoResize = false,
                    allowToggleVisibility = false
                },
                new MultiColumnHeaderState.Column
                {
                    headerContent = new GUIContent("OnDemandDownload"),
                    headerTextAlignment = TextAlignment.Left,
                    sortedAscending = true,
                    sortingArrowAlignment = TextAlignment.Center,
                    width = 150,
                    minWidth = 50,
                    autoResize = false,
                    allowToggleVisibility = false
                },
                new MultiColumnHeaderState.Column
                {
                    headerContent = new GUIContent("AB"),
                    headerTextAlignment = TextAlignment.Left,
                    sortedAscending = true,
                    sortingArrowAlignment = TextAlignment.Center,
                    width = 80,
                    minWidth = 50,
                    autoResize = false,
                    allowToggleVisibility = false
                },
            };

            // Assert.AreEqual(columns.Length, Enum.GetValues(typeof(MyColumns)).Length, "Number of columns should match number of enum values: You probably forgot to update one of them.");

            var state = new MultiColumnHeaderState(columns);
            return state;
        }

        protected override void RowGUI(RowGUIArgs args)
        {
            var item = (TreeViewItemBaseT<ASSceneTreeDataItem>)args.item;

            for (int i = 0; i < args.GetNumVisibleColumns(); ++i)
            {
                CellGUI(args.GetCellRect(i), item, (MyColumns)args.GetColumn(i), ref args);
            }
        }

        void CellGUI(Rect cellRect, TreeViewItemBaseT<ASSceneTreeDataItem> item, MyColumns column, ref RowGUIArgs args)
        {
            // Center cell rect vertically (makes it easier to place controls, icons etc in the cells)
            CenterRectUsingSingleLineHeight(ref cellRect);

            switch (column)
            {
                case MyColumns.AssetPath:
                {
                    string value = item.Data.SceneSettings.assetPath;
                    DefaultGUI.Label(cellRect, value, args.selected, args.focused);
                }
                break;
                case MyColumns.OnDemandDownload:
                {
                    // Do toggle
                    Rect toggleRect = cellRect;
                    toggleRect.width = k_ToggleWidth;
                    if (toggleRect.xMax < cellRect.xMax)
                    {
                        bool isEnabled = EditorGUI.Toggle(toggleRect, item.Data.SceneSettings.onDemandDownload);
                        if (isEnabled != item.Data.SceneSettings.onDemandDownload)
                        {
                            IList<int> ids = GetSelection();
                            if (!ids.Contains(item.id))
                            {
                                var tmpSettings = item.Data.SceneSettings;
                                tmpSettings.onDemandDownload = isEnabled;
                            }
                            else
                            {
                                List<ASSceneTreeDataItem> elems = ASMainWindow.Instance.SceneData;
                                foreach (int id in ids)
                                {
                                    var tmpSettings = elems[id].SceneSettings;
                                    tmpSettings.onDemandDownload = isEnabled;
                                }
                            }
                        }
                    }
                }
                break;
                case MyColumns.AssetBundle:
                {
                    string value = EditorUtility.FormatBytes(item.Data.SceneSettings.assetBundleSize);
                    DefaultGUI.Label(cellRect, value, args.selected, args.focused);
                }
                break;
            }
        }
    }
}

#endif  // IG_C111 || IG_C301, Auto generated by AddMacroForInstantGameFiles.exe
